/* ********** err.c - errors and warnings ********** */

#include "lvb.h"

#ifdef IAMDANIEL	/* write full but poss. out of date message on crash */
static void logserr(void);
#endif	/* ifdef IAMDANIEL */

void crash(const char *const fmt, ...)
/* log dire warning followed by vfprintf-acceptable message supplied, and if
 * IAMDANIEL is defined also log system error message if any;
 * exit abnormally */
{
	const char *const warning = "FATAL ERROR";	/* dire warning */
	extern FILE *logfp;				/* output file */
	va_list args;					/* arguments */

	va_start(args, fmt);
	fprintf(logfp, "%s: ", warning);
	vfprintf(logfp, fmt, args);
	va_end(args);
	fputc('\n', logfp);

#ifdef IAMDANIEL	/* log sys. error message that may not be up to date */
	logserr();
#endif	/* ifdef IAMDANIEL */

	exit(1);

}	/* end crash() */

void scream(const char *const fmt, ...)
/* log a dire warning, partly composed of vfprintf-acceptable user-supplied
 * message */
{
	const char *const warning = "ERROR";
	extern FILE *logfp;	/* output file */
	va_list args;		/* supplied message */

	va_start(args, fmt);
	fprintf(logfp, "%s: ", warning);
	vfprintf(logfp, fmt, args);
	va_end(args);
	fputc('\n', logfp);

	/* flush log file so the warning is immediately visible */
	if (fflush(logfp) == EOF)
		crash("write error on log file");	/* msg may not work! */

}	/* end scream() */

#ifdef IAMDANIEL
static void logserr(void)
/* log system error that may be out of date */
{
	extern FILE *logfp;	/* output file */
	extern int errno;	/* error */

	if (errno != 0)
		fprintf(logfp, "System error '%s'\n", strerror(errno));
	else
		fprintf(logfp, "Error 0\n");
	return;

}	/* end logserr() */
#endif	/* ifdef IAMDANIEL */
