/* ********** sops.c - string operations ********** */

#include "lvb.h"

enum { SAME, DIFFERENT };	/* strings considered same or different */

Integer strcicmp(const char *const s1, const char *const s2)
/* compare strings s1 and s2, whose length may be different,
 * in a case-insensitive way;
 * return 0 if they are the same, non-zero if different */
{
	Integer i;	/* loop counter */
	Integer len1;	/* length of s1 */
	Integer len2;	/* length of s2 */

	len1 = (Integer) strlen(s1);
	len2 = (Integer) strlen(s2);
	
	if (len1 != len2)	/* can't be identical */
		return (Integer) DIFFERENT;

	for (i = 0; i < len1; ++i)
	{
		if (toupper((int) s1[i]) != toupper((int) s2[i]))
			return (Integer) DIFFERENT;
	}
	return (Integer) SAME;

}	/* end strcicmp() */

char *nextnonwspc(const char *string)
/* return pointer to next non-white space character in string, or NULL if
 * none */
{
        while (isspace(*string))
                ++string;
	if (*string)
                return (char *) string;
        else
                return NULL;

}	/* end nextnonwspc() */

char *salloc(const Integer len, const char *const msg)
/* return space for string len letters long (i.e. len + 1 bytes);
 * on failure, crash with message "out of memory: cannot allocate for "
 * followed by msg */
{
	char *s;	/* new space */
        
	s = (char *) alloc((size_t) (len + 1), msg);
	return s;
	
}       /* end salloc() */

char *supper(char *const s)
/* make all letters in string s upper case;
 * return s */
{
	char *elementptr = s;

	while (*elementptr)
	{
		*elementptr = (char) toupper((int) *elementptr);
		++elementptr;
	}
	return s;

}	/* end supper() */
